
#include<iostream>
#include<sstream>
#include<string>
#include<vector>
#include<map>
using namespace std;



map<int, map<char, int> > CreateAutomata(string str) {
	map<int, map<char, int> > delta;
	vector<bool> current(str.size()+1,false);
	current[0] = true;
	vector<bool> next(str.size()+1,false);
	next[0] = true;

	for(int i = 0; i < str.size(); i++) {
		// make transitions for state i (depending on which states the NFA would
		// be in after reading w := str[0]..str[i-1]
		for(int j = 0; j < current.size(); j++)
			if(current[j]) {
				// current[j] == true  iff  the NFA can be in j after reading w
				delta[i][str[j]] = j+1;
				if(str[j] == str[i])
					next[j+1] = true;
			}
		current = next;
		next = vector<bool>(str.size(),false);
		next[0] = true;
	}
	return delta;
}

bool AutomataAccepts(map<int,map<char,int> >& delta, istream& is) {
	int state = 0;
	char readchar;
	while(true) {
		if(is.peek() == -1)
			return false;
		readchar = is.get();

		if(delta[state].find(readchar) == delta[state].end()) {
			state = 0;
		} else {
			state = delta[state][readchar];
			if(state == delta.size())
				return true;
		}
	}
}

int main(int argc, char** argv) {
	if(argc <= 2) {
		cerr << "usage: "<<argv[0]<<" str1 str2\n";
		cerr << "       the prog then checks, if str1 occurs in str2 by creating\n";
		cerr << "       a deterministic automata for Sigma* str1 Sigma* and\n";
		cerr << "       applying it to str2. The Automata is constructed directly,\n";
		cerr << "       not by using the powerset-construction on the Thompson-\n";
		cerr << "       Construction of str1.\n";
		return 1;
	}


	map<int, map<char,int> > Automata = CreateAutomata(argv[1]);


	cout << "\nThe Automata\n----------------------";
	for(map<int,map<char,int> >::iterator it = Automata.begin(); it != Automata.end(); it++) {
		cout << "\nstate " << it->first << "\n";
		for(map<char,int>::iterator jt = it->second.begin(); jt != it->second.end(); jt++) {
			cout << "   --" << jt->first << "--> " << jt->second << "\n";
		}
	}
	cout << "----------------------\n\n";


	stringstream str(argv[2]);
	cout << "string \"" << argv[1] << "\" " << (AutomataAccepts(Automata, str) ? "" : "not ")
	     << "found in \"" << argv[2] << "\"\n\n";
	return 0;
}
